@@ -25,7 +25,6 @@ from .helpers import BasicAuth
 
 __all__ = ('BaseConnector', 'TCPConnector', 'ProxyConnector', 'UnixConnector')
 
-PY_341 = sys.version_info >= (3, 4, 1)
 PY_343 = sys.version_info >= (3, 4, 3)
 
 HASHFUNC_BY_DIGESTLEN = {
@@ -53,24 +52,26 @@ class Connection(object):
         if loop.get_debug():
             self._source_traceback = traceback.extract_stack(sys._getframe(1))
 
-    if PY_341:
-        def __del__(self):
-            if self._transport is not None:
-                if hasattr(self._loop, 'is_closed'):
-                    if self._loop.is_closed():
-                        return
+    def __repr__(self):
+        return 'Connection<{}>'.format(self._key)
 
-                self._connector._release(
-                    self._key, self._request, self._transport, self._protocol,
-                    should_close=True)
+    def __del__(self, _warnings=warnings):
+        if self._transport is not None:
+            _warnings.warn('Unclosed connection {!r}'.format(self),
+                           ResourceWarning)
+            if hasattr(self._loop, 'is_closed'):
+                if self._loop.is_closed():
+                    return
 
-                warnings.warn("Unclosed connection {!r}".format(self),
-                              ResourceWarning)
-                context = {'client_connection': self,
-                           'message': 'Unclosed connection'}
-                if self._source_traceback is not None:
-                    context['source_traceback'] = self._source_traceback
-                self._loop.call_exception_handler(context)
+            self._connector._release(
+                self._key, self._request, self._transport, self._protocol,
+                should_close=True)
+
+            context = {'client_connection': self,
+                       'message': 'Unclosed connection'}
+            if self._source_traceback is not None:
+                context['source_traceback'] = self._source_traceback
+            self._loop.call_exception_handler(context)
 
     @property
     def loop(self):
@@ -122,7 +123,7 @@ class BaseConnector(object):
             self._source_traceback = traceback.extract_stack(sys._getframe(1))
 
         self._conns = {}
-        self._acquired = defaultdict(list)
+        self._acquired = defaultdict(set)
         self._conn_timeout = conn_timeout
         self._keepalive_timeout = keepalive_timeout
         if share_cookies:
@@ -142,22 +143,24 @@ class BaseConnector(object):
 
         self.cookies = http.cookies.SimpleCookie()
 
-    if PY_341:
-        def __del__(self):
-            if self._closed:
-                return
-            if not self._conns:
-                return
+    def __del__(self, _warnings=warnings):
+        if self._closed:
+            return
+        if not self._conns:
+            return
 
-            self.close()
+        conns = [repr(c) for c in self._conns.values()]
 
-            warnings.warn("Unclosed connector {!r}".format(self),
-                          ResourceWarning)
-            context = {'connector': self,
-                       'message': 'Unclosed connector'}
-            if self._source_traceback is not None:
-                context['source_traceback'] = self._source_traceback
-            self._loop.call_exception_handler(context)
+        self.close()
+
+        _warnings.warn("Unclosed connector {!r}".format(self),
+                       ResourceWarning)
+        context = {'connector': self,
+                   'connections': conns,
+                   'message': 'Unclosed connector'}
+        if self._source_traceback is not None:
+            context['source_traceback'] = self._source_traceback
+        self._loop.call_exception_handler(context)
 
     @property
     def force_close(self):
@@ -274,12 +277,24 @@ class BaseConnector(object):
         """Get from pool or create new connection."""
         key = (req.host, req.port, req.ssl)
 
-        # use short-circuit
-        if self._limit is not None:
-            while len(self._acquired[key]) >= self._limit:
-                fut = asyncio.Future(loop=self._loop)
-                self._waiters[key].append(fut)
-                yield from fut
+        limit = self._limit
+        if limit is not None:
+            fut = asyncio.Future(loop=self._loop)
+            waiters = self._waiters[key]
+
+            # The limit defines the maximum number of concurrent connections
+            # for a key. Waiters must be counted against the limit, even before
+            # the underlying connection is created.
+            available = limit - len(waiters) - len(self._acquired[key])
+
+            # Don't wait if there are connections available.
+            if available > 0:
+                fut.set_result(None)
+
+            # This connection will now count towards the limit.
+            waiters.append(fut)
+
+            yield from fut
 
         transport, proto = self._get(key)
         if transport is None:
@@ -291,25 +306,25 @@ class BaseConnector(object):
                 else:
                     transport, proto = yield from self._create_connection(req)
 
-                if not self._force_close:
-                    if self._conns.get(key, None) is None:
-                        self._conns[key] = []
-
-                    self._conns[key].append((transport, proto,
-                                            self._loop.time()))
             except asyncio.TimeoutError as exc:
                 raise ClientTimeoutError(
-                    'Connection timeout to host %s:%s ssl:%s' % key) from exc
+                    'Connection timeout to host {0[0]}:{0[1]} ssl:{0[2]}'
+                    .format(key)) from exc
             except OSError as exc:
                 raise ClientOSError(
-                    'Cannot connect to host %s:%s ssl:%s' % key) from exc
+                    exc.errno,
+                    'Cannot connect to host {0[0]}:{0[1]} ssl:{0[2]} [{1}]'
+                    .format(key, exc.strerror)) from exc
 
-        self._acquired[key].append(transport)
+        self._acquired[key].add(transport)
         conn = Connection(self, key, req, transport, proto, self._loop)
         return conn
 
     def _get(self, key):
-        conns = self._conns.get(key)
+        try:
+            conns = self._conns[key]
+        except KeyError:
+            return None, None
         t1 = self._loop.time()
         while conns:
             transport, proto, t0 = conns.pop()
@@ -318,8 +333,12 @@ class BaseConnector(object):
                     transport.close()
                     transport = None
                 else:
+                    if not conns:
+                        # The very last connection was reclaimed: drop the key
+                        del self._conns[key]
                     return transport, proto
-
+        # No more connections: drop the key
+        del self._conns[key]
         return None, None
 
     def _release(self, key, req, transport, protocol, *, should_close=False):
@@ -330,7 +349,7 @@ class BaseConnector(object):
         acquired = self._acquired[key]
         try:
             acquired.remove(transport)
-        except ValueError:  # pragma: no cover
+        except KeyError:  # pragma: no cover
             # this may be result of undetermenistic order of objects
             # finalization due garbage collection.
             pass
@@ -346,26 +365,13 @@ class BaseConnector(object):
         resp = req.response
 
         if not should_close:
-            if resp is not None:
-                if resp.message is None:
-                    should_close = True
-                else:
-                    should_close = resp.message.should_close
-
             if self._force_close:
                 should_close = True
+            elif resp is not None:
+                should_close = resp._should_close
 
         reader = protocol.reader
         if should_close or (reader.output and not reader.output.at_eof()):
-            conns = self._conns.get(key)
-            if conns is not None and len(conns) >= 0:
-                # Issue #253: An empty array will eventually be
-                # removed by cleanup, but it's better to pop straight
-                # away, because cleanup might not get called (e.g. if
-                # keepalive is False).
-                if not acquired:
-                    self._conns.pop(key, None)
-
             transport.close()
         else:
             conns = self._conns.get(key)
@@ -382,7 +388,6 @@ class BaseConnector(object):
 
 
 _SSL_OP_NO_COMPRESSION = getattr(ssl, "OP_NO_COMPRESSION", 0)
-_SSH_HAS_CREATE_DEFAULT_CONTEXT = hasattr(ssl, 'create_default_context')
 
 _marker = object()
 
@@ -403,7 +408,7 @@ class TCPConnector(BaseConnector):
 
     def __init__(self, *, verify_ssl=True, fingerprint=None,
                  resolve=_marker, use_dns_cache=_marker,
-                 family=socket.AF_INET, ssl_context=None,
+                 family=0, ssl_context=None,
                  **kwargs):
         super().__init__(**kwargs)
 
@@ -466,17 +471,8 @@ class TCPConnector(BaseConnector):
                 sslcontext.options |= ssl.OP_NO_SSLv3
                 sslcontext.options |= _SSL_OP_NO_COMPRESSION
                 sslcontext.set_default_verify_paths()
-            elif _SSH_HAS_CREATE_DEFAULT_CONTEXT:
-                # Python 3.4+
-                sslcontext = ssl.create_default_context()
             else:
-                # Fallback for Python 3.3.
-                sslcontext = ssl.SSLContext(ssl.PROTOCOL_SSLv23)
-                sslcontext.options |= ssl.OP_NO_SSLv2
-                sslcontext.options |= ssl.OP_NO_SSLv3
-                sslcontext.options |= _SSL_OP_NO_COMPRESSION
-                sslcontext.set_default_verify_paths()
-                sslcontext.verify_mode = ssl.CERT_REQUIRED
+                sslcontext = ssl.create_default_context()
             self._ssl_context = sslcontext
         return self._ssl_context
 
@@ -580,6 +576,11 @@ class TCPConnector(BaseConnector):
                 has_cert = transp.get_extra_info('sslcontext')
                 if has_cert and self._fingerprint:
                     sock = transp.get_extra_info('socket')
+                    if not hasattr(sock, 'getpeercert'):
+                        # Workaround for asyncio 3.5.0
+                        # Starting from 3.5.1 version
+                        # there is 'ssl_object' extra info in transport
+                        sock = transp._ssl_protocol._sslpipe.ssl_object
                     # gives DER-encoded cert as a sequence of bytes (or None)
                     cert = sock.getpeercert(binary_form=True)
                     assert cert
@@ -592,8 +593,9 @@ class TCPConnector(BaseConnector):
             except OSError as e:
                 exc = e
         else:
-            raise ClientOSError('Can not connect to %s:%s' %
-                                (req.host, req.port)) from exc
+            raise ClientOSError(exc.errno,
+                                'Can not connect to %s:%s [%s]' %
+                                (req.host, req.port, exc.strerror)) from exc
 
 
 class ProxyConnector(TCPConnector):
@@ -655,7 +657,10 @@ class ProxyConnector(TCPConnector):
         if hdrs.AUTHORIZATION in proxy_req.headers:
             auth = proxy_req.headers[hdrs.AUTHORIZATION]
             del proxy_req.headers[hdrs.AUTHORIZATION]
-            req.headers[hdrs.PROXY_AUTHORIZATION] = auth
+            if not req.ssl:
+                req.headers[hdrs.PROXY_AUTHORIZATION] = auth
+            else:
+                proxy_req.headers[hdrs.PROXY_AUTHORIZATION] = auth
 
         if req.ssl:
             # For HTTPS requests over HTTP proxy
@@ -672,7 +677,7 @@ class ProxyConnector(TCPConnector):
             key = (req.host, req.port, req.ssl)
             conn = Connection(self, key, proxy_req,
                               transport, proto, self._loop)
-            self._acquired[key].append(conn._transport)
+            self._acquired[key].add(conn._transport)
             proxy_resp = proxy_req.send(conn.writer, conn.reader)
             try:
                 resp = yield from proxy_resp.start(conn, True)
@@ -692,6 +697,8 @@ class ProxyConnector(TCPConnector):
                 transport, proto = yield from self._loop.create_connection(
                     self._factory, ssl=self.ssl_context, sock=rawsock,
                     server_hostname=req.host)
+            finally:
+                proxy_resp.close()
 
         return transport, proto
 
