@@ -165,6 +165,17 @@ class MultipartResponseWrapper(object):
         self.resp = resp
         self.stream = stream
 
+    @asyncio.coroutine
+    def __aiter__(self):
+        return self
+
+    @asyncio.coroutine
+    def __anext__(self):
+        part = yield from self.next()
+        if part is None:
+            raise StopAsyncIteration  # NOQA
+        return part
+
     def at_eof(self):
         """Returns ``True`` when all response data had been read.
 
@@ -202,6 +213,17 @@ class BodyPartReader(object):
         self._read_bytes = 0
         self._unread = deque()
 
+    @asyncio.coroutine
+    def __aiter__(self):
+        return self
+
+    @asyncio.coroutine
+    def __anext__(self):
+        part = yield from self.next()
+        if part is None:
+            raise StopAsyncIteration  # NOQA
+        return part
+
     @asyncio.coroutine
     def next(self):
         item = yield from self.read()
@@ -430,6 +452,17 @@ class MultipartReader(object):
         self._at_eof = False
         self._unread = []
 
+    @asyncio.coroutine
+    def __aiter__(self):
+        return self
+
+    @asyncio.coroutine
+    def __anext__(self):
+        part = yield from self.next()
+        if part is None:
+            raise StopAsyncIteration  # NOQA
+        return part
+
     @classmethod
     def from_response(cls, response):
         """Constructs reader instance from HTTP response.
@@ -625,9 +658,9 @@ class BodyPartWriter(object):
         """Yields byte chunks for body part."""
 
         has_encoding = (
-            CONTENT_ENCODING in self.headers
-            and self.headers[CONTENT_ENCODING] != 'identity'
-            or CONTENT_TRANSFER_ENCODING in self.headers
+            CONTENT_ENCODING in self.headers and
+            self.headers[CONTENT_ENCODING] != 'identity' or
+            CONTENT_TRANSFER_ENCODING in self.headers
         )
         if has_encoding:
             # since we're following streaming approach which doesn't assumes
@@ -702,9 +735,10 @@ class BodyPartWriter(object):
         if encoding == 'identity':
             yield from stream
         elif encoding in ('deflate', 'gzip'):
-            zlib_mode = (16 + zlib.MAX_WBITS
-                         if encoding == 'gzip' else
-                         -zlib.MAX_WBITS)
+            if encoding == 'gzip':
+                zlib_mode = 16 + zlib.MAX_WBITS
+            else:
+                zlib_mode = -zlib.MAX_WBITS
             zcomp = zlib.compressobj(wbits=zlib_mode)
             for chunk in stream:
                 yield zcomp.compress(chunk)
@@ -755,11 +789,9 @@ class BodyPartWriter(object):
                     raise ValueError('bad content disposition parameter'
                                      ' {!r}={!r}'.format(key, val))
                 qval = quote(val, '')
+                lparams.append((key, '"%s"' % qval))
                 if key == 'filename':
-                    lparams.append((key, '"%s"' % qval))
                     lparams.append(('filename*', "utf-8''" + qval))
-                else:
-                    lparams.append((key, "%s" % qval))
             sparams = '; '.join('='.join(pair) for pair in lparams)
             value = '; '.join((value, sparams))
         self.headers[CONTENT_DISPOSITION] = value
