@@ -3,17 +3,18 @@
 import collections
 import functools
 import http.server
-import itertools
 import re
 import string
 import sys
 import zlib
+from abc import abstractmethod, ABCMeta
 from wsgiref.handlers import format_date_time
 
 import aiohttp
 from . import errors, hdrs
-from .multidict import CIMultiDict
+from .multidict import CIMultiDict, upstr
 from .log import internal_logger
+from .helpers import reify
 
 __all__ = ('HttpMessage', 'Request', 'Response',
            'HttpVersion', 'HttpVersion10', 'HttpVersion11',
@@ -475,7 +476,7 @@ def filter_pipe(filter, filter2, *,
         chunk = yield EOL_MARKER
 
 
-class HttpMessage:
+class HttpMessage(metaclass=ABCMeta):
     """HttpMessage allows to write headers and payload to a stream.
 
     For example, lets say we want to read file then compress it with deflate
@@ -526,8 +527,6 @@ class HttpMessage:
     SERVER_SOFTWARE = 'Python/{0[0]}.{0[1]} aiohttp/{1}'.format(
         sys.version_info, aiohttp.__version__)
 
-    status = None
-    status_line = b''
     upgrade = False  # Connection: UPGRADE
     websocket = False  # Upgrade: WEBSOCKET
     has_chunked_hdr = False  # Transfer-encoding: chunked
@@ -538,7 +537,7 @@ class HttpMessage:
 
     def __init__(self, transport, version, close):
         self.transport = transport
-        self.version = version
+        self._version = version
         self.closing = close
         self.keepalive = None
         self.chunked = False
@@ -546,8 +545,26 @@ class HttpMessage:
         self.headers = CIMultiDict()
         self.headers_sent = False
         self.output_length = 0
+        self.headers_length = 0
         self._output_size = 0
 
+    @property
+    @abstractmethod
+    def status_line(self):
+        return b''
+
+    @abstractmethod
+    def autochunked(self):
+        return False
+
+    @property
+    def version(self):
+        return self._version
+
+    @property
+    def body_length(self):
+        return self.output_length - self.headers_length
+
     def force_close(self):
         self.closing = True
         self.keepalive = False
@@ -585,7 +602,7 @@ class HttpMessage:
             'Header {!r} should have string value, got {!r}'.format(
                 name, value)
 
-        name = name.strip().upper()
+        name = upstr(name)
         value = value.strip()
 
         if name == hdrs.CONTENT_LENGTH:
@@ -628,9 +645,7 @@ class HttpMessage:
         assert not self.headers_sent, 'headers have been sent already'
         self.headers_sent = True
 
-        if self.chunked or (self.length is None and
-                            self.version >= HttpVersion11 and
-                            self.status not in (304, 204)):
+        if self.chunked or self.autochunked():
             self.writer = self._write_chunked_payload()
             self.headers[hdrs.TRANSFER_ENCODING] = 'chunked'
 
@@ -645,12 +660,12 @@ class HttpMessage:
         self._add_default_headers()
 
         # status + headers
-        headers = ''.join(itertools.chain(
-            (self.status_line,),
-            *((k, _sep, v, _end) for k, v in self.headers.items())))
+        headers = self.status_line + ''.join(
+            [k + _sep + v + _end for k, v in self.headers.items()])
         headers = headers.encode('utf-8') + b'\r\n'
 
         self.output_length += len(headers)
+        self.headers_length = len(headers)
         self.transport.write(headers)
 
     def _add_default_headers(self):
@@ -723,9 +738,7 @@ class HttpMessage:
 
             chunk = bytes(chunk)
             chunk_len = '{:x}\r\n'.format(len(chunk)).encode('ascii')
-            self.transport.write(chunk_len)
-            self.transport.write(chunk)
-            self.transport.write(b'\r\n')
+            self.transport.write(chunk_len + chunk + b'\r\n')
             self.output_length += len(chunk_len) + len(chunk) + 2
 
     def _write_length_payload(self, length):
@@ -811,8 +824,8 @@ class Response(HttpMessage):
     HOP_HEADERS = ()
 
     @staticmethod
-    def calc_reason(status):
-        record = RESPONSES.get(status)
+    def calc_reason(status, *, _RESPONSES=RESPONSES):
+        record = _RESPONSES.get(status)
         if record is not None:
             reason = record[0]
         else:
@@ -823,13 +836,29 @@ class Response(HttpMessage):
                  http_version=HttpVersion11, close=False, reason=None):
         super().__init__(transport, http_version, close)
 
-        self.status = status
+        self._status = status
         if reason is None:
             reason = self.calc_reason(status)
 
-        self.reason = reason
-        self.status_line = 'HTTP/{}.{} {} {}\r\n'.format(
-            http_version[0], http_version[1], status, reason)
+        self._reason = reason
+
+    @property
+    def status(self):
+        return self._status
+
+    @property
+    def reason(self):
+        return self._reason
+
+    @reify
+    def status_line(self):
+        version = self.version
+        return 'HTTP/{}.{} {} {}\r\n'.format(
+            version[0], version[1], self.status, self.reason)
+
+    def autochunked(self):
+        return (self.length is None and
+                self.version >= HttpVersion11)
 
     def _add_default_headers(self):
         super()._add_default_headers()
@@ -853,12 +882,23 @@ class Request(HttpMessage):
 
         super().__init__(transport, http_version, close)
 
-        self.method = method
-        self.path = path
-        self.status_line = '{0} {1} HTTP/{2[0]}.{2[1]}\r\n'.format(
-            method, path, http_version)
+        self._method = method
+        self._path = path
 
-    def _add_default_headers(self):
-        super()._add_default_headers()
+    @property
+    def method(self):
+        return self._method
 
-        self.headers.setdefault(hdrs.USER_AGENT, self.SERVER_SOFTWARE)
+    @property
+    def path(self):
+        return self._path
+
+    @reify
+    def status_line(self):
+        return '{0} {1} HTTP/{2[0]}.{2[1]}\r\n'.format(
+            self.method, self.path, self.version)
+
+    def autochunked(self):
+        return (self.length is None and
+                self.version >= HttpVersion11 and
+                self.status not in (304, 204))
