@@ -5,13 +5,10 @@ import asyncio
 import gc
 import unittest
 import unittest.mock
-import sys
 
 import aiohttp
 from aiohttp.client_reqrep import ClientResponse
 
-PY_341 = sys.version_info >= (3, 4, 1)
-
 
 class TestClientResponse(unittest.TestCase):
 
@@ -21,19 +18,23 @@ class TestClientResponse(unittest.TestCase):
 
         self.connection = unittest.mock.Mock()
         self.stream = aiohttp.StreamParser(loop=self.loop)
-        self.response = ClientResponse('get', 'http://python.org')
+        self.response = ClientResponse('get', 'http://def-cl-resp.org')
         self.response._post_init(self.loop)
+        self.response._setup_connection(self.connection)
 
     def tearDown(self):
+        self.response.close()
         self.loop.close()
+        gc.collect()
 
-    @unittest.skipUnless(PY_341, "Requires Python 3.4.1+")
     def test_del(self):
-        response = ClientResponse('get', 'http://python.org')
+        response = ClientResponse('get', 'http://del-cl-resp.org')
         response._post_init(self.loop)
 
         connection = unittest.mock.Mock()
         response._setup_connection(connection)
+        self.loop.set_exception_handler(lambda loop, ctx: None)
+
         with self.assertWarns(ResourceWarning):
             del response
             gc.collect()
@@ -44,25 +45,28 @@ class TestClientResponse(unittest.TestCase):
         self.response._connection = self.connection
         self.response.close()
         self.assertIsNone(self.response.connection)
-        self.assertTrue(self.connection.release.called)
         self.response.close()
         self.response.close()
 
-    def test_wait_for_100(self):
+    def test_wait_for_100_1(self):
         response = ClientResponse(
             'get', 'http://python.org', continue100=object())
         response._post_init(self.loop)
         self.assertTrue(response.waiting_for_continue())
+        response.close()
+
+    def test_wait_for_100_2(self):
         response = ClientResponse(
             'get', 'http://python.org')
         response._post_init(self.loop)
         self.assertFalse(response.waiting_for_continue())
+        response.close()
 
     def test_repr(self):
         self.response.status = 200
         self.response.reason = 'Ok'
         self.assertIn(
-            '<ClientResponse(http://python.org) [200 Ok]>',
+            '<ClientResponse(http://def-cl-resp.org) [200 Ok]>',
             repr(self.response))
 
     def test_read_and_release_connection(self):
@@ -72,42 +76,29 @@ class TestClientResponse(unittest.TestCase):
             return fut
         content = self.response.content = unittest.mock.Mock()
         content.read.side_effect = side_effect
-        self.response.close = unittest.mock.Mock()
 
         res = self.loop.run_until_complete(self.response.read())
         self.assertEqual(res, b'payload')
-        self.assertTrue(self.response.close.called)
+        self.assertIsNone(self.response._connection)
 
     def test_read_and_release_connection_with_error(self):
         content = self.response.content = unittest.mock.Mock()
         content.read.return_value = asyncio.Future(loop=self.loop)
         content.read.return_value.set_exception(ValueError)
-        self.response.close = unittest.mock.Mock()
 
         self.assertRaises(
             ValueError,
             self.loop.run_until_complete, self.response.read())
-        self.response.close.assert_called_with(True)
+        self.assertTrue(self.response._closed)
 
     def test_release(self):
         fut = asyncio.Future(loop=self.loop)
         fut.set_result(b'')
         content = self.response.content = unittest.mock.Mock()
         content.readany.return_value = fut
-        self.response.close = unittest.mock.Mock()
 
         self.loop.run_until_complete(self.response.release())
-        self.assertTrue(self.response.close.called)
-
-    def test_read_and_close(self):
-        self.response.read = unittest.mock.Mock()
-        self.response.read.return_value = asyncio.Future(loop=self.loop)
-        self.response.read.return_value.set_result(b'data')
-
-        with self.assertWarns(DeprecationWarning):
-            res = self.loop.run_until_complete(self.response.read_and_close())
-        self.assertEqual(res, b'data')
-        self.assertTrue(self.response.read.called)
+        self.assertIsNone(self.response._connection)
 
     def test_read_decode_deprecated(self):
         self.response._content = b'data'
@@ -129,11 +120,10 @@ class TestClientResponse(unittest.TestCase):
             'CONTENT-TYPE': 'application/json;charset=cp1251'}
         content = self.response.content = unittest.mock.Mock()
         content.read.side_effect = side_effect
-        self.response.close = unittest.mock.Mock()
 
         res = self.loop.run_until_complete(self.response.text())
         self.assertEqual(res, '{"тест": "пройден"}')
-        self.assertTrue(self.response.close.called)
+        self.assertIsNone(self.response._connection)
 
     def test_text_custom_encoding(self):
         def side_effect(*args, **kwargs):
@@ -144,13 +134,12 @@ class TestClientResponse(unittest.TestCase):
             'CONTENT-TYPE': 'application/json'}
         content = self.response.content = unittest.mock.Mock()
         content.read.side_effect = side_effect
-        self.response.close = unittest.mock.Mock()
         self.response._get_encoding = unittest.mock.Mock()
 
         res = self.loop.run_until_complete(
             self.response.text(encoding='cp1251'))
         self.assertEqual(res, '{"тест": "пройден"}')
-        self.assertTrue(self.response.close.called)
+        self.assertIsNone(self.response._connection)
         self.assertFalse(self.response._get_encoding.called)
 
     def test_text_detect_encoding(self):
@@ -161,12 +150,11 @@ class TestClientResponse(unittest.TestCase):
         self.response.headers = {'CONTENT-TYPE': 'application/json'}
         content = self.response.content = unittest.mock.Mock()
         content.read.side_effect = side_effect
-        self.response.close = unittest.mock.Mock()
 
         self.loop.run_until_complete(self.response.read())
         res = self.loop.run_until_complete(self.response.text())
         self.assertEqual(res, '{"тест": "пройден"}')
-        self.assertTrue(self.response.close.called)
+        self.assertIsNone(self.response._connection)
 
     def test_text_after_read(self):
         def side_effect(*args, **kwargs):
@@ -177,11 +165,10 @@ class TestClientResponse(unittest.TestCase):
             'CONTENT-TYPE': 'application/json;charset=cp1251'}
         content = self.response.content = unittest.mock.Mock()
         content.read.side_effect = side_effect
-        self.response.close = unittest.mock.Mock()
 
         res = self.loop.run_until_complete(self.response.text())
         self.assertEqual(res, '{"тест": "пройден"}')
-        self.assertTrue(self.response.close.called)
+        self.assertIsNone(self.response._connection)
 
     def test_json(self):
         def side_effect(*args, **kwargs):
@@ -192,11 +179,10 @@ class TestClientResponse(unittest.TestCase):
             'CONTENT-TYPE': 'application/json;charset=cp1251'}
         content = self.response.content = unittest.mock.Mock()
         content.read.side_effect = side_effect
-        self.response.close = unittest.mock.Mock()
 
         res = self.loop.run_until_complete(self.response.json())
         self.assertEqual(res, {'тест': 'пройден'})
-        self.assertTrue(self.response.close.called)
+        self.assertIsNone(self.response._connection)
 
     def test_json_custom_loader(self):
         self.response.headers = {
@@ -214,7 +200,6 @@ class TestClientResponse(unittest.TestCase):
         self.response.headers = {
             'CONTENT-TYPE': 'data/octet-stream'}
         self.response._content = b''
-        self.response.close = unittest.mock.Mock()
 
         res = self.loop.run_until_complete(self.response.json())
         self.assertIsNone(res)
@@ -231,13 +216,12 @@ class TestClientResponse(unittest.TestCase):
             'CONTENT-TYPE': 'application/json;charset=utf8'}
         content = self.response.content = unittest.mock.Mock()
         content.read.side_effect = side_effect
-        self.response.close = unittest.mock.Mock()
         self.response._get_encoding = unittest.mock.Mock()
 
         res = self.loop.run_until_complete(
             self.response.json(encoding='cp1251'))
         self.assertEqual(res, {'тест': 'пройден'})
-        self.assertTrue(self.response.close.called)
+        self.assertIsNone(self.response._connection)
         self.assertFalse(self.response._get_encoding.called)
 
     def test_json_detect_encoding(self):
@@ -248,16 +232,15 @@ class TestClientResponse(unittest.TestCase):
         self.response.headers = {'CONTENT-TYPE': 'application/json'}
         content = self.response.content = unittest.mock.Mock()
         content.read.side_effect = side_effect
-        self.response.close = unittest.mock.Mock()
 
         res = self.loop.run_until_complete(self.response.json())
         self.assertEqual(res, {'тест': 'пройден'})
-        self.assertTrue(self.response.close.called)
+        self.assertIsNone(self.response._connection)
 
     def test_override_flow_control(self):
         class MyResponse(ClientResponse):
             flow_control_class = aiohttp.FlowControlDataQueue
-        response = MyResponse('get', 'http://python.org')
+        response = MyResponse('get', 'http://my-cl-resp.org')
         response._post_init(self.loop)
         response._setup_connection(self.connection)
         self.assertIsInstance(response.content, aiohttp.FlowControlDataQueue)
@@ -269,3 +252,9 @@ class TestClientResponse(unittest.TestCase):
 
         self.response.headers = {'CONTENT-TYPE': 'application/json'}
         self.assertEqual(self.response._get_encoding(), 'utf-8')
+
+    def test_close_deprecated(self):
+        self.response._connection = self.connection
+        with self.assertWarns(DeprecationWarning):
+            self.response.close(force=False)
+        self.assertIsNone(self.response._connection)
